/**
 * $RCSfile: ,v $
 * $Revision: $
 * $Date: $
 *
 * Copyright (C) 2004-2011 Jive Software. All rights reserved.
 * Custom modifications (C) 2014, 2018 by SilentVault.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * NB: alignment is a bit of a mess, but best viewed with tabsize=4
 */

package org.jivesoftware;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.AffineTransform;
import java.io.File;
import java.io.IOException;
import java.security.Principal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.net.InetSocketAddress;
import java.net.InetAddress;
import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;
import java.net.HttpURLConnection;
import java.net.ProtocolException;
import java.net.UnknownHostException;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.login.Configuration;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JRadioButton;
import javax.swing.JComboBox;
import javax.swing.JButton;
import javax.swing.ButtonGroup;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JPopupMenu;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.JSeparator;
import javax.swing.Box;
import javax.swing.text.JTextComponent;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.jivesoftware.resource.Default;
import org.jivesoftware.resource.Res;
import org.jivesoftware.resource.SparkRes;
import org.jivesoftware.resource.ConfigurationRes;
import org.jivesoftware.smack.ConnectionConfiguration;
import org.jivesoftware.smack.proxy.ProxyInfo;
import org.jivesoftware.smack.SASLAuthentication;
import org.jivesoftware.smack.SmackConfiguration;
import org.jivesoftware.smack.XMPPConnection;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smack.packet.XMPPError;
import org.jivesoftware.smack.util.StringUtils;
import org.jivesoftware.smackx.ChatStateManager;
import org.jivesoftware.spark.SessionManager;
import org.jivesoftware.spark.SparkManager;
import org.jivesoftware.spark.Workspace;
import org.jivesoftware.spark.component.RolloverButton;
import org.jivesoftware.spark.util.BrowserLauncher;
import org.jivesoftware.spark.util.DummySSLSocketFactory;
import org.jivesoftware.spark.util.GraphicUtils;
import org.jivesoftware.spark.util.ModelUtil;
import org.jivesoftware.spark.util.ResourceUtils;
import org.jivesoftware.spark.util.SwingWorker;
import org.jivesoftware.spark.util.log.Log;
import org.jivesoftware.sparkimpl.settings.JiveInfo;
import org.jivesoftware.sparkimpl.plugin.layout.LayoutSettings;
import org.jivesoftware.sparkimpl.plugin.layout.LayoutSettingsManager;
import org.jivesoftware.sparkimpl.settings.local.LocalPreferences;
import org.jivesoftware.sparkimpl.settings.local.SettingsManager;

/**
 * Dialog to log in a user into the Spark Server. The LoginDialog is used only
 * for login in registered users into the Spark Server.
 */
public class LoginDialog {
    private JFrame loginDialog;
    private static final String BUTTON_PANEL = "buttonpanel"; // NOTRANS
    private static final String PROGRESS_BAR = "progressbar"; // NOTRANS
    private LocalPreferences localPref;
    private ArrayList<String> _usernames = new ArrayList<String>();
    private String loginUsername;
    private String loginPassword;
    private String loginServer;

    /**
     * whether or not the network connect is internal/private or external/public
     */
    private boolean	internalNet;

    /**
     * the gateway we're to connect to, in private network mode
     */
    private OFSPathDescriptor	gtwyConfig;

    /**
     * the login panel object
     */
    protected LoginPanel loginPanel;

    /**
     * Empty Constructor
     */
    public LoginDialog() {
        localPref = SettingsManager.getLocalPreferences();

        // Check if upgraded needed.
        try {
            checkForOldSettings();
        }
        catch (Exception e) {
            Log.error(e);
        }
    }

    /**
     * Invokes the LoginDialog to be visible.
     *
     * @param parentFrame the parentFrame of the Login Dialog. This is used
     *                    for correct parenting.
     */
    public void invoke(final JFrame parentFrame) {
        // Before creating any connections. Update proxy if needed.
        try {
            updateProxyConfig();
        }
        catch (Exception e) {
            Log.error(e);
        }

       // Construct Dialog
    	 EventQueue.invokeLater(new Runnable() {
   		 public void run() {
   	        loginDialog = new JFrame(Default.getString(Default.APPLICATION_NAME));
   	        loginDialog.setIconImage(SparkManager.getApplicationImage().getImage());
   	        loginPanel = new LoginPanel();
   	        loginPanel.setOpaque(false);
   	        final JPanel mainPanel = new LoginBackgroundPanel();
   	        final GridBagLayout mainLayout = new GridBagLayout();
   	        mainPanel.setLayout(mainLayout);

   	        final ImagePanel imagePanel = new ImagePanel();

   	        mainPanel.add(imagePanel,
   	                new GridBagConstraints(0, 0, 4, 1,
   	                        1.0, 0.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
   	                        new Insets(0, 0, 0, 0), 0, 0));

   	        final String showPoweredBy = Default.getString(Default.SHOW_POWERED_BY);
   	        if (ModelUtil.hasLength(showPoweredBy) && "true".equals(showPoweredBy)) {
   	            // Handle Powered By for custom clients.
   	            final JLabel poweredBy = new JLabel(SparkRes.getImageIcon(SparkRes.POWERED_BY_IMAGE));
   	            mainPanel.add(poweredBy,
   	                    new GridBagConstraints(0, 1, 4, 1,
   	                            1.0, 0.0, GridBagConstraints.NORTHEAST, GridBagConstraints.HORIZONTAL,
   	                            new Insets(0, 0, 3, 0), 0, 0));

   	        }

   	        mainPanel.add(loginPanel,
   	                new GridBagConstraints(0, 2, 2, 1,
   	                        1.0, 1.0, GridBagConstraints.NORTHWEST, GridBagConstraints.BOTH,
   	                        new Insets(5, 5, 5, 5), 0, 0));

   	        loginDialog.setContentPane(mainPanel);
   	        loginDialog.setLocationRelativeTo(parentFrame);

   	        loginDialog.setResizable(true);
   	        loginDialog.pack();

   	        // Center dialog on screen
   	        GraphicUtils.centerWindowOnScreen(loginDialog);

   	        // Show dialog
   	        loginDialog.addWindowListener(new WindowAdapter() {
   	            public void windowClosing(WindowEvent e) {
   	                quitLogin();
   	            }
   	        });
   	        if (loginPanel.getUsername().trim().length() > 0) {
   	            loginPanel.getPasswordField().requestFocus();
   	        }

   	        if (!localPref.isStartedHidden() || !localPref.isAutoLogin()) {
   	            // Make dialog top most.
   	            loginDialog.setVisible(true);
   	        }
   		 }
   	 });
    }

    //This method can be overwritten by subclasses to provide additional validations
    //(such as certificate download functionality when connecting)
    protected boolean beforeLoginValidations() {
	/* The following is a hack for supporting OFS hosts which are PageKites.
	 * A PageKite requires an open HTTP connection from the same incoming IP
	 * in order to proxy an encrypted connection across a raw port.  We
	 * therefore open '/' on the host first, before initiating XMPP connect.
	 * However, in the event that the OFShost isn't a PageKite and doesn't
	 * have anything listening on port 80, we will not let a failure abort
	 * our connect attempt.  (This is actually the case with the test net.)
	 */
	boolean ret = false;
	if (!getInternalNet()) {
		// nothing to do here
		ret = true;
		return ret;
	}
       OFSPathDescriptor internalConfig = getGtwyConfig();
       if (internalConfig == null) {
	       Log.error("Missing internal network config");
	       return ret;
       }
       try {
	       URL pkURL = new URL("http", internalConfig.m_AccessPath.getHostString(), 80, "/");
	       URLConnection pkConn = pkURL.openConnection();
	       HttpURLConnection preConn = (HttpURLConnection) pkConn;
	       preConn.setRequestMethod("GET");
	       preConn.setAllowUserInteraction(false);
	       preConn.setDoOutput(true);
	       preConn.setConnectTimeout(10000);
	       preConn.setReadTimeout(10000);
	       preConn.connect();
	       Object junk = preConn.getContent();
	       preConn.disconnect();
	       ret = true;
       }
       catch (MalformedURLException mue) {
		Log.error("Could not construct URL for pre-connect step", mue);
       }
       catch (ProtocolException pe) {
	       Log.error("Bad protocol for connect to URL for pre-connect step", pe);
       }
       catch (IOException ioe) {
	       Log.error("Could not connect to URL for pre-connect step");
       }
       return ret;
    }
    
    protected void afterLogin() {
        // Does noting by default - but can be overwritten by subclasses to provide additional
        // settings
    }    

    protected ConnectionConfiguration retrieveConnectionConfiguration() {
        int port = localPref.getXmppPort();

        int checkForPort = loginServer.indexOf(":");
        if (checkForPort != -1) {
            String portString = loginServer.substring(checkForPort + 1);
            if (ModelUtil.hasLength(portString)) {
                // Set new port.
                port = Integer.valueOf(portString);
            }
        }

        boolean useSSL = localPref.isSSL();
        boolean hostPortConfigured = localPref.isHostAndPortConfigured();

        ConnectionConfiguration config = null;

        ProxyInfo proxyInfo = null;
        if (localPref.isProxyEnabled()) {
        	ProxyInfo.ProxyType pType = localPref.getProtocol().equals("SOCKS") ?
        		ProxyInfo.ProxyType.SOCKS5 : ProxyInfo.ProxyType.HTTP;
        	String pHost = ModelUtil.hasLength(localPref.getHost()) ?
        		localPref.getHost() : null;
        	int pPort = ModelUtil.hasLength(localPref.getPort()) ?
        		Integer.parseInt(localPref.getPort()) : 0;
        	String pUser = ModelUtil.hasLength(localPref.getProxyUsername()) ?
        		localPref.getProxyUsername() : null;
        	String pPass = ModelUtil.hasLength(localPref.getProxyPassword()) ?
        		localPref.getProxyPassword() : null;
        	
        	if (pHost != null && pPort != 0) {
                   
        		if (pUser == null || pPass == null) {
                                
        			proxyInfo = new ProxyInfo(pType, pHost, pPort, null, null);
        		} else {
                               
        			proxyInfo = new ProxyInfo(pType, pHost, pPort, pUser, pPass);
                                                                       
        		}
        	} else {
        		Log.error("No proxy info found but proxy type is enabled!");
        	}
        }
        
        if (useSSL) {
            if (!hostPortConfigured) {
                config = new ConnectionConfiguration(loginServer, 5223);
                config.setSocketFactory(new DummySSLSocketFactory());
            }
            else {
                config = new ConnectionConfiguration(localPref.getXmppHost(), port, loginServer);
                config.setSocketFactory(new DummySSLSocketFactory());
            }
            if(localPref.isProxyEnabled() &&  !hostPortConfigured)
            {
                
                config = new ConnectionConfiguration(loginServer,5223,proxyInfo);
            }
            else if(localPref.isProxyEnabled() &&  !hostPortConfigured)
            {
                
                config = new ConnectionConfiguration(localPref.getXmppHost(), port, loginServer,proxyInfo);
            
            }
        }
        else {
            if(!localPref.isProxyEnabled())
            {
            if (!hostPortConfigured) {
                config = new ConnectionConfiguration(loginServer);
            }
            else {
                
                config = new ConnectionConfiguration(localPref.getXmppHost(), port, loginServer);
            }
            }
            else
            {
                 if (!hostPortConfigured) {
                     
                config = new ConnectionConfiguration(loginServer,proxyInfo);
            }
            else {
                
                config = new ConnectionConfiguration(localPref.getXmppHost(), port, loginServer,proxyInfo);
            }
            
            }

        }
        config.setReconnectionAllowed(true);
        config.setRosterLoadedAtLogin(true);
        config.setSendPresence(false);

        if (localPref.isPKIEnabled()) {
            SASLAuthentication.supportSASLMechanism("EXTERNAL");
            config.setKeystoreType(localPref.getPKIStore());
            if(localPref.getPKIStore().equals("PKCS11")) {
                config.setPKCS11Library(localPref.getPKCS11Library());
            }
            else if(localPref.getPKIStore().equals("JKS")) {
                config.setKeystoreType("JKS");
                config.setKeystorePath(localPref.getJKSPath());

            }
            else if(localPref.getPKIStore().equals("X509")) {
                //do something
            }
            else if(localPref.getPKIStore().equals("Apple")) {
                config.setKeystoreType("Apple");
            }
        }

        boolean compressionEnabled = localPref.isCompressionEnabled();
        config.setCompressionEnabled(compressionEnabled);            
        if(ModelUtil.hasLength(localPref.getTrustStorePath())) {
        	config.setTruststorePath(localPref.getTrustStorePath());
        	config.setTruststorePassword(localPref.getTrustStorePassword());
        }
        return config;
    }
    
    /**
     * Define Login Panel implementation.
     */
   public final class LoginPanel extends JPanel
		implements KeyListener, ActionListener, FocusListener, CallbackHandler
   {
       private static final long serialVersionUID = 2445523786538863459L;

       /* SV customizations: we must first select whether the connection is
	* going to be internal/private or external/public.  In the former case,
	* the gateway connected to will be accessed via a PageKite.  This
	* requires prior HTTP access on host:80 to establish the connection
	* tunnel.  The list of host:post gateways is specified by the
	* gateway.properties.  In the latter case, a server and port can be
	* entered by the user and the usual XMPP connection will be utilized.
	*
	* Next we determine whether the connection is to be anonymous or
	* identified.  If the former (the default for private networks) then
	* there is no username/password required.  Otherwise, permit usual
	* username/password entries and allow the stored "other users" access.
	*/
       private final JRadioButton privRadio = new JRadioButton();
       private final JLabel gtwyLabel = new JLabel();
       private final JComboBox<String> gtwyList = new JComboBox<String>();
       private final JRadioButton externalRadio = new JRadioButton();
       private final ButtonGroup networkGroup = new ButtonGroup();
       private final JRadioButton anonRadio = new JRadioButton();
       private final JRadioButton identRadio = new JRadioButton();
       private final ButtonGroup identGroup = new ButtonGroup();
       private final JCheckBox testNetBox = new JCheckBox();
		 private final JLabel defaultLabel = new JLabel();

	  // definition of normal Spark UI objects
	  private final JLabel usernameLabel = new JLabel();
      private final JTextField usernameField = new JTextField();

      private final JLabel passwordLabel = new JLabel();
      private final JPasswordField passwordField = new JPasswordField();

      private final JLabel serverLabel = new JLabel();
      private final JTextField serverField = new JTextField();

      private final JCheckBox savePasswordBox = new JCheckBox();
      private final JCheckBox autoLoginBox = new JCheckBox();
      private final JButton loginButton = new JButton();
      private final JButton advancedButton = new JButton();
      private final JButton quitButton = new JButton();
	  private final JCheckBox loginAsInvisibleBox = new JCheckBox();

      private final JButton createAccountButton = new JButton();
      private final RolloverButton passwordResetButton = new RolloverButton();

      private final JLabel progressBar = new JLabel();

      // Panel used to hold buttons
      private final CardLayout cardLayout = new CardLayout(0, 5);
      final JPanel cardPanel = new JPanel(cardLayout);

      final JPanel buttonPanel = new JPanel(new GridBagLayout());
      private final GridBagLayout GRIDBAGLAYOUT = new GridBagLayout();

      private JLabel headerLabel = new JLabel();
      private JLabel accountLabel = new JLabel();
      private JLabel accountNameLabel = new JLabel();
      private JLabel serverNameLabel = new JLabel();
      private JLabel ssoServerLabel = new JLabel();

      private RolloverButton otherUsers = new RolloverButton(SparkRes.getImageIcon(SparkRes.PANE_UP_ARROW_IMAGE));

      private XMPPConnection connection = null;

	  /**
	   * available hostnames for defined SilentVault OFS gateways
	   */
       private ArrayList<OFSPathDescriptor>	m_AvailOFSHosts;

	  /**
	   * the index of the selected gateway in m_AvailOFSHosts
	   */
       private int				m_OFSHostIndex;

	  /**
	   * hostname of the OFS (currently selected from paths in m_AvailOFSHosts)
	   */
       private String				m_OFShost;

	  /**
	   * port number of the OFS (selected from paths in m_AvailOFSHosts)
	   */
       private int				m_OFSport;

	  /**
	   * XMPP domain name of the OFS (currently selected from paths in
	   * m_AvailOFSHosts)
	   */
       private String				m_OFSdomain;

	  /**
	   * nullary constructor
	   */
       LoginPanel() {
	    // set default button group selections and control states
	    networkGroup.add(privRadio);
	    networkGroup.add(externalRadio);
	    privRadio.setSelected(true);
	    identGroup.add(anonRadio);
	    identGroup.add(identRadio);
	    anonRadio.setSelected(true);
	    gtwyList.setEnabled(true);
	    testNetBox.setSelected(false);
	    serverField.setEnabled(false);
	    advancedButton.setEnabled(false);
	    usernameField.setEnabled(false);
	    passwordField.setEnabled(false);
	    otherUsers.setEnabled(false);
	    createAccountButton.setEnabled(false);
	    savePasswordBox.setEnabled(false);
	    loginButton.setEnabled(true);

		// init generic Spark controls
        ResourceUtils.resButton(savePasswordBox, Res.getString("checkbox.save.password"));
	    savePasswordBox.setToolTipText("Remember passwords for usernames");
        ResourceUtils.resButton(autoLoginBox, Res.getString("checkbox.auto.login"));
        ResourceUtils.resLabel(serverLabel, serverField, "External Jabber " + Res.getString("label.server") + ":");
        ResourceUtils.resButton(createAccountButton, Res.getString("label.accounts"));
	    createAccountButton.setToolTipText("Register a new username on "
	    					+ "the server");
        ResourceUtils.resButton(passwordResetButton, Res.getString("label.passwordreset"));
	    ResourceUtils.resButton(loginAsInvisibleBox, Res.getString("checkbox.login.as.invisible"));
	    loginAsInvisibleBox.setToolTipText("Connect with invisible presence");

	      // init SV-specific controls
	      ResourceUtils.resButton(privRadio, "Private Network");
	      privRadio.setToolTipText("Connect to the SilentVault network "
	      				+ "(wallet access)");
	      ResourceUtils.resButton(externalRadio, "Public Network");
	      externalRadio.setToolTipText("Connect to any public XMPP server "
	      				+ "(no wallet access)");
			ResourceUtils.resLabel(defaultLabel, this,
				"For regular wallet access, select any Private Network Access "
				+ "Gateway and click on Login.");
	      ResourceUtils.resLabel(gtwyLabel, gtwyList, "Access Gateway:");
	      ResourceUtils.resButton(anonRadio, "Be Anonymous");
	      anonRadio.setToolTipText("Your username will be a random hex "
	      				+ "number");
	      ResourceUtils.resButton(identRadio, "Use Username");
	      identRadio.setToolTipText("Specify a chat handle of your choosing");
	      ResourceUtils.resButton(testNetBox, "Test Network");
	      testNetBox.setToolTipText("Connect to the test network instead of live");
	      testNetBox.setOpaque(false);
	      privRadio.setOpaque(false);
	      externalRadio.setOpaque(false);
	      anonRadio.setOpaque(false);
	      gtwyList.setOpaque(false);
	      anonRadio.setOpaque(false);
	      identRadio.setOpaque(false);
	      m_AvailOFSHosts = new ArrayList<OFSPathDescriptor>();

	      // give a default in case spark.properties is lacking definitions
	      m_OFShost = "ofs.voucher-safe.org";
	      m_OFSport = 5233;
	      m_OFSdomain = "ofs.vm.to";

	      // populate gtwyList from definitions in spark.properties
	      loadGateways();
	      for (int ofsIdx = 0; ofsIdx < m_AvailOFSHosts.size(); ofsIdx++) {
		      OFSPathDescriptor ofsAddr = m_AvailOFSHosts.get(ofsIdx);
		      String gtwyItem = ofsAddr.m_AccessPath.getHostName()
					+ ":" + ofsAddr.m_AccessPath.getPort();
		      gtwyList.addItem(gtwyItem);
	      }
	      gtwyList.setToolTipText("Select a host:port from the list");

         savePasswordBox.setOpaque(false);
         autoLoginBox.setOpaque(false);
	    	loginAsInvisibleBox.setOpaque(false);
         setLayout(GRIDBAGLAYOUT);

         // Set default visibility
         headerLabel.setVisible(false);
         accountLabel.setVisible(false);
         accountNameLabel.setVisible(false);
         serverNameLabel.setVisible(false);
         headerLabel.setText(Res.getString("title.advanced.connection.sso"));
         headerLabel.setFont(headerLabel.getFont().deriveFont(Font.BOLD));
         accountLabel.setText("Account:");
         ssoServerLabel.setText("Server:");
         accountNameLabel.setFont(accountLabel.getFont().deriveFont(Font.BOLD));
         serverNameLabel.setFont(ssoServerLabel.getFont().deriveFont(Font.BOLD));
         accountNameLabel.setForeground(new Color(106, 127, 146));
         serverNameLabel.setForeground(new Color(106, 127, 146));
         otherUsers.setFocusable(false);
	    	otherUsers.setToolTipText("Select from among stored user profiles");

	   // now place UI controls into layout
	   GridBagConstraints gbc = new GridBagConstraints();
	   gbc.anchor = GridBagConstraints.WEST;
	   final Insets stdInset = new Insets(5, 10, 5, 10);
	   gbc.insets = stdInset;
	   gbc.gridwidth = GridBagConstraints.REMAINDER;
	   gbc.fill = GridBagConstraints.HORIZONTAL;
	   gbc.gridheight = 12;

		// row 0: default instructions label
		add(defaultLabel, gbc);

	   // row 1: private network radio, gateway label, gateway selector
	   JPanel gtwyPane = new JPanel(new GridLayout(1, 3));
	   gtwyPane.setOpaque(false);
	   gtwyPane.add(privRadio);
	   gtwyPane.add(gtwyLabel);
	   gtwyPane.add(gtwyList);
	   add(gtwyPane, gbc);
	   privRadio.addActionListener(this);
	   gtwyList.addActionListener(this);
	   JSeparator sep1 = new JSeparator();
	   add(sep1, gbc);

	   // row 2: external network radio, server label, server
	   JPanel serverPane = new JPanel(new GridLayout(1, 3));
	   serverPane.setOpaque(false);
	   serverPane.add(externalRadio);
	   serverPane.add(serverLabel);
	   serverPane.add(serverField);
	   add(serverPane, gbc);
	   externalRadio.addActionListener(this);

	   // row 3: spacer
	   JSeparator sep2 = new JSeparator();
	   add(sep2, gbc);

	   // row 4: anon radio, spacer, identified radio
	   JPanel anonPane = new JPanel(new GridLayout(1, 3));
	   anonPane.setOpaque(false);
	   anonPane.add(anonRadio);
	   anonRadio.addActionListener(this);
	   anonPane.add(identRadio);
	   anonPane.add(Box.createHorizontalStrut(150));
	   identRadio.addActionListener(this);
	   add(anonPane, gbc);
	   
	   // row 5: username label, username field, other (stored) users icon
	   JPanel userPane = new JPanel(new GridLayout(1, 3));
	   userPane.setOpaque(false);
      userPane.add(usernameLabel);
      userPane.add(usernameField);
      userPane.add(otherUsers);
	   add(userPane, gbc);

	   // row 6: password label, password field, spacer
	   JPanel passPane = new JPanel(new GridLayout(1, 3));
	   passPane.setOpaque(false);
      passPane.add(passwordLabel);
      passPane.add(passwordField);
	   passPane.add(Box.createHorizontalStrut(150));
	   add(passPane, gbc);

	   // row 7: test net, save password, login invisible checkboxes
	   JPanel checkPane = new JPanel(new GridLayout(1, 3));
	   checkPane.setOpaque(false);
	   checkPane.add(testNetBox);
      checkPane.add(savePasswordBox);
	   checkPane.add(loginAsInvisibleBox);
	   add(checkPane, gbc);
	   testNetBox.addActionListener(this);
      autoLoginBox.addActionListener(this);
      savePasswordBox.addActionListener(this);
	   loginAsInvisibleBox.addActionListener(this);
	   JSeparator sep3 = new JSeparator();
	   add(sep3, gbc);

	    // prepare button panel
	   final Insets buttonInset = new Insets(5, 20, 5, 20);
        if (!Default.getBoolean(Default.ACCOUNT_DISABLED)) {
        	buttonPanel.add(createAccountButton,
                        new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
                        GridBagConstraints.EAST, GridBagConstraints.NONE, buttonInset, 0, 0));
            }

        if (Default.getBoolean(Default.PASSWORD_RESET_ENABLED)) {
        	buttonPanel.add(passwordResetButton, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, buttonInset, 0, 0));
            passwordResetButton.addActionListener(new ActionListener() {
                final String url = Default.getString(Default.PASSWORD_RESET_URL);
                private static final long serialVersionUID = 2680369963282231348L;

                public void actionPerformed(ActionEvent actionEvent) {
				   try {
					   BrowserLauncher.openURL(url);
				   }
				   catch (Exception e) {
					   Log.error("Unable to load password reset.", e);
				   }
                }
            });
        }

            if (!Default.getBoolean(Default.ADVANCED_DISABLED)){
            	buttonPanel.add(advancedButton,
                    new GridBagConstraints(2, 0, 1, 1, 1.0, 0.0,
                            GridBagConstraints.EAST, GridBagConstraints.NONE, buttonInset, 0, 0));
            }
			advancedButton.setToolTipText("Specify advanced connection "
														+ "preferences on public servers");
			loginButton.setToolTipText("Click to establish server connection");
            buttonPanel.add(loginButton,
                    new GridBagConstraints(3, 0, 1, 1, 1.0, 0.0,
                            GridBagConstraints.EAST, GridBagConstraints.NONE, buttonInset, 0, 0));

            cardPanel.add(buttonPanel, BUTTON_PANEL);

            cardPanel.setOpaque(false);
            buttonPanel.setOpaque(false);

            ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource("images/ajax-loader.gif"));
            progressBar.setIcon(icon);
            cardPanel.add(progressBar, PROGRESS_BAR);


		    // row 8: card panel used to hold buttons and login progress bar
		    gbc.anchor = GridBagConstraints.SOUTH;
          add(cardPanel, gbc);

            // Add KeyListener
            usernameField.addKeyListener(this);
            passwordField.addKeyListener(this);
            serverField.addKeyListener(this);

            passwordField.addFocusListener(this);
            usernameField.addFocusListener(this);
            serverField.addFocusListener(this);

            // Add ActionListener
            quitButton.addActionListener(this);
            loginButton.addActionListener(this);
            advancedButton.addActionListener(this);

            otherUsers.addMouseListener(new MouseAdapter() {
					@Override
					public void mouseClicked(MouseEvent e) {
						getPopup().show(otherUsers, e.getX(), e.getY());
					}
	    	});

            // make username and password fields same size
            //GraphicUtils.makeSameSize(usernameField, passwordField);

            // Set progress bar description
            progressBar.setText(Res.getString("message.autenticating"));
            progressBar.setVerticalTextPosition(JLabel.BOTTOM);
            progressBar.setHorizontalTextPosition(JLabel.CENTER);
            progressBar.setHorizontalAlignment(JLabel.CENTER);

            // Set more Resources
            ResourceUtils.resLabel(usernameLabel, usernameField, Res.getString("label.username") + ":");
            ResourceUtils.resLabel(passwordLabel, passwordField, Res.getString("label.password") + ":");
            ResourceUtils.resButton(quitButton, Res.getString("button.quit"));
            ResourceUtils.resButton(loginButton, Res.getString("button.login"));
            ResourceUtils.resButton(advancedButton, Res.getString("button.advanced"));

            // Load previous instances
            String userProp = localPref.getLastUsername();
            String serverProp = localPref.getServer();

           File file = new File(Spark.getSparkUserHome(), "/user/");
           File[] userprofiles = file.listFiles();

	    for (File f : userprofiles) {

		if (f.getName().contains("@")) {
		    _usernames.add(f.getName());
		} else {
		    Log.error("Profile contains wrong format: \"" + f.getName()
			    + "\" located at: " + f.getAbsolutePath());
		}

	    }


            if (userProp != null) {
                usernameField.setText(StringUtils.unescapeNode(userProp));
            }
            if (serverProp != null) {
                serverField.setText(serverProp);
                serverNameLabel.setText(serverProp);
            }

            // Check Settings
            if (localPref.isSavePassword()) {

                String encryptedPassword = localPref.getPasswordForUser(getBareJid());
                if (encryptedPassword != null) {
                    passwordField.setText(encryptedPassword);
                }
                savePasswordBox.setSelected(true);
                loginButton.setEnabled(true);
            }
            //autoLoginBox.setSelected(localPref.isAutoLogin());
            autoLoginBox.setSelected(false);
	    		loginAsInvisibleBox.setSelected(localPref.isLoginAsInvisible());
            useSSO(localPref.isSSOEnabled());
			/*
			 * disable auto-logins
            if (autoLoginBox.isSelected()) {
                savePasswordBox.setEnabled(false);
                autoLoginBox.setEnabled(false);
					 loginAsInvisibleBox.setEnabled(false);
                validateLogin();
                return;
            }
			 */

            // Handle arguments
            String username = Spark.getArgumentValue("username");
            String password = Spark.getArgumentValue("password");
            String server = Spark.getArgumentValue("server");

            if (username != null) {
                usernameField.setText(username);
            }

            if (password != null) {
                passwordField.setText(password);
            }

            if (server != null) {
                serverField.setText(server);
            }

            if (username != null && server != null && password != null) {
                savePasswordBox.setEnabled(false);
                autoLoginBox.setEnabled(false);
					 loginAsInvisibleBox.setEnabled(false);
                validateLogin();
            }

            createAccountButton.addActionListener(this);

            final String lockedDownURL = Default.getString(Default.HOST_NAME);
            if (ModelUtil.hasLength(lockedDownURL)) {
                serverField.setText(lockedDownURL);
            }
            if (Default.getBoolean("HOST_NAME_CHANGE_DISABLED"))
                serverField.setEnabled(false);
        }

        /**
         * Returns the username the user defined.
         *
         * @return the username.
         */
        private String getUsername() {
            return StringUtils.escapeNode(usernameField.getText().trim());
        }

        /**
         * Returns the resulting bareJID from username and server
         * @return
         */
        private String getBareJid()
        {
            return usernameField.getText()+"@"+serverField.getText();
        }

        /**
         * Returns the password specified by the user.
         *
         * @return the password.
         */
        private String getPassword() {
            return new String(passwordField.getPassword());
        }

        /**
         * Returns the server name specified by the user.
         *
         * @return the server name.
         */
        private String getServerName() {
            return serverField.getText().trim();
        }

		/**
         * Return whether user wants to login as invisible or not.
         *
         * @return the true if user wants to login as invisible.
         */
         private boolean isLoginAsInvisible() {
            return loginAsInvisibleBox.isSelected();
         }

	   /**
		* obtain whether user wants to connect to the test network
		* @return true if test net wanted
		*/
	   public boolean isTestNet() {
		  return testNetBox.isSelected();
	   }

	 /**
	  * obtain whether network is internal or external
	  * @return true if internal
	  */
	 public boolean isInternal() {
		 return privRadio.isSelected();
	 }

	 /**
	  * obtain whether the login should be anonymous
	  * @return true if anon login requested
	  */
	 private boolean isAnonymous() {
		 return anonRadio.isSelected();
	 }

	 /**
	  * obtain the gateway config selected by the user
	  * @return the gateway OFS configuration, based on the control index
	  */
	 private OFSPathDescriptor getGatewayConfig() {
		 return m_AvailOFSHosts.get(m_OFSHostIndex);
	 }

	 /**
	  * obtain the current OFS hostname (PageKite host)
	  * @return the host
	  */
	 public String getOFShost() { return m_OFShost; }
	 
	 /**
	  * obtain the current OFS port (PageKite port)
	  * @return the port
	  */
	 public int getOFSport() { return m_OFSport; }

	 /**
	  * obtain the OFS XMPP domain name
	  * @return the domain
	  */
	 public String getOFSdomain() { return m_OFSdomain; }

	 /**
	  * ActionListener implementation.
	  *
	  * @param e the ActionEvent
	  */
	 public void actionPerformed(ActionEvent e) {
	 Object src = e.getSource();
		 if (src == quitButton) {
			 quitLogin();
		 }
		 else if (src == createAccountButton) {
			 AccountCreationWizard createAccountPanel
					 = new AccountCreationWizard(loginPanel);
			 createAccountPanel.invoke(loginDialog);

			 if (createAccountPanel.isRegistered()) {
				 usernameField.setText(createAccountPanel.getUsernameWithoutEscape());
				 passwordField.setText(createAccountPanel.getPassword());
				 serverField.setText(createAccountPanel.getServer());
				 loginButton.setEnabled(true);
			 }
		 }
		 else if (src == loginButton) {
			 validateLogin();
		 }
		 else if (src == advancedButton) {
			 final LoginSettingDialog loginSettingsDialog = new LoginSettingDialog();
			 loginSettingsDialog.invoke(loginDialog);
			 useSSO(localPref.isSSOEnabled());
		 }
		 else if (src == savePasswordBox) {
			 autoLoginBox.setEnabled(savePasswordBox.isSelected());

			 if (!savePasswordBox.isSelected()) {
				 autoLoginBox.setSelected(false);
			 }
		 }
		 else if (src == autoLoginBox) {
			 if (autoLoginBox.isSelected()) {
				 savePasswordBox.setSelected(true);
			 }
		 }
		  else if (src == privRadio) {
			  if (privRadio.isSelected()) {
				  gtwyList.setEnabled(true);
				  serverField.setEnabled(false);
				  advancedButton.setEnabled(false);
				  anonRadio.setToolTipText("Your username will be "
						   + "a random hex number");
			  }
		  }
		  else if (src == externalRadio) {
			  if (externalRadio.isSelected()) {
				  serverField.setEnabled(true);
				  gtwyList.setEnabled(false);
				  advancedButton.setEnabled(true);
				  anonRadio.setToolTipText("Many public servers may "
						   + "not allow this");
				  testNetBox.setSelected(false);
			  }
		  }
		  else if (src == anonRadio) {
			  if (anonRadio.isSelected()) {
				  usernameField.setEnabled(false);
				  passwordField.setEnabled(false);
				  savePasswordBox.setEnabled(false);
				  otherUsers.setEnabled(false);
				  createAccountButton.setEnabled(false);
				  if (externalRadio.isSelected()) {
					  // warn that public networks might not allow this
					  anonRadio.setToolTipText("Many public servers may not allow this");
				  }
				  else {
					  anonRadio.setToolTipText("Your username will be a random hex number");
				  }
			  }
		  }
		  else if (src == identRadio) {
			  if (identRadio.isSelected()) {
				  usernameField.setEnabled(true);
				  passwordField.setEnabled(true);
				  savePasswordBox.setEnabled(true);
				  otherUsers.setEnabled(true);
				  createAccountButton.setEnabled(true);
			  }
		  }
		  else if (src == gtwyList) {
			  m_OFSHostIndex = gtwyList.getSelectedIndex();
			  OFSPathDescriptor curPath = m_AvailOFSHosts.get(m_OFSHostIndex);
			  m_OFShost = curPath.m_AccessPath.getHostString();
			  m_OFSport = curPath.m_AccessPath.getPort();
			  m_OFSdomain = curPath.m_XMPPDomain;
			  if (m_OFSHostIndex == m_AvailOFSHosts.size()-1) {
				 // this is the test network, so check the box
				 testNetBox.setSelected(true);
			  }
			  else {
				 // anything else is not the test network, so uncheck the box
				 testNetBox.setSelected(false);
			  }
		  }
		  else if (src == testNetBox) {
			  if (isTestNet()) {
				externalRadio.setSelected(false);
				privRadio.setSelected(true);
				advancedButton.setEnabled(false);
				serverField.setEnabled(false);
				// set the last gateway; this will be the test network
				gtwyList.setSelectedIndex(m_AvailOFSHosts.size()-1);
			  }
			  else {
				  // set the first gateway; this will be the live network
				  gtwyList.setSelectedIndex(0);
			  }
			  gtwyList.setEnabled(true);
		  }
	   }

        private JPopupMenu getPopup() {
            JPopupMenu popup = new JPopupMenu();
	    for(final String key : _usernames) {

		JMenuItem menu = new JMenuItem(key);

		final String username = key.split("@")[0];
		final String host = key.split("@")[1];
		menu.addActionListener(new ActionListener() {

		    @Override
		    public void actionPerformed(ActionEvent e) {
			usernameField.setText(username);
			serverField.setText(host);

			try {
			    passwordField.setText(localPref.getPasswordForUser(getBareJid()));
			    if (passwordField.getPassword().length < 1
			        && !isAnonymous())
			    {
				//loginButton.setEnabled(false);
			    }
			    else {
				loginButton.setEnabled(true);
			    }
			} catch (Exception e1) {
			}

		    }
		});

		popup.add(menu);
	    }
	    return popup;
        }

        /**
         * KeyListener implementation.
         *
         * @param e the KeyEvent to process.
         */
        public void keyTyped(KeyEvent e) {
            validate(e);
        }

        public void keyPressed(KeyEvent e) {
            if(e.getKeyCode() == KeyEvent.VK_RIGHT &&
        	    ((JTextField)e.getSource()).getCaretPosition()==((JTextField)e.getSource()).getText().length())
            {
        	getPopup().show(otherUsers,0,0);
            }
        }

        public void keyReleased(KeyEvent e) {
            validateDialog();
        }

        /**
         * Checks the users input and enables/disables the login button depending on state.
         */
        private void validateDialog() {
			 /*
			  * since we can log in anonymously, always want login button enabled
            loginButton.setEnabled(
                    ModelUtil.hasLength(getUsername()) &&
                    ( ModelUtil.hasLength(getPassword()) || localPref.isSSOEnabled() ) &&
                    ModelUtil.hasLength(getServerName())   );
			  */
            loginButton.setEnabled(true);
        }

        /**
         * Validates key input.
         *
         * @param e the keyEvent.
         */
        private void validate(KeyEvent e) {
            if (loginButton.isEnabled() && e.getKeyChar() == KeyEvent.VK_ENTER) {
                validateLogin();
            }
        }

        public void focusGained(FocusEvent e) {
            Object o = e.getSource();
            if (o instanceof JTextComponent) {
                ((JTextComponent)o).selectAll();
            }
        }

        public void focusLost(FocusEvent e) {
        }

        /**
         * Enables/Disables the editable components in the login screen.
         *
         * @param editable true to enable components, otherwise false to disable.
         */
        private void enableComponents(boolean editable) {

            // Need to set both editable and enabled for best behavior.
            usernameField.setEditable(editable);
            usernameField.setEnabled(editable);

            passwordField.setEditable(editable);
            passwordField.setEnabled(editable);

            final String lockedDownURL = Default.getString(Default.HOST_NAME);
            if (!ModelUtil.hasLength(lockedDownURL)) {
                serverField.setEditable(editable);
                serverField.setEnabled(editable);
            }


            if (editable) {
                // Reapply focus to username field
                passwordField.requestFocus();
            }
        }

        /**
         * Displays the progress bar.
         *
         * @param visible true to display progress bar, false to hide it.
         */
        private void setProgressBarVisible(boolean visible) {
            if (visible) {
                cardLayout.show(cardPanel, PROGRESS_BAR);
                // progressBar.setIndeterminate(true);
            }
            else {
                cardLayout.show(cardPanel, BUTTON_PANEL);
            }
        }

        /**
         * Validates the users login information.
         */
        private void validateLogin() {
            final SwingWorker loginValidationThread = new SwingWorker() {
                public Object construct() {
                    setLoginUsername(getUsername());
                    setLoginPassword(getPassword());
                    setLoginServer(getServerName());
		    setInternalNet(isInternal());
		    setGtwyConfig(getGatewayConfig());
                    boolean loginSuccessfull = beforeLoginValidations() && login();
                    if (loginSuccessfull) {
                        afterLogin();
                        progressBar.setText(Res.getString("message.connecting.please.wait"));

                        // Startup Spark
                        startSpark();

                        // dispose login dialog
                        loginDialog.dispose();

                        // Show ChangeLog if we need to.
                        // new ChangeLogDialog().showDialog();
                    }
                    else {
                        EventQueue.invokeLater(new Runnable() {
                            @Override
                            public void run() {
                                savePasswordBox.setEnabled(true);
                                autoLoginBox.setEnabled(true);
										  loginAsInvisibleBox.setVisible(true);
                                enableComponents(true);
                                setProgressBarVisible(false);
                            }
                        });

                    }
                    return loginSuccessfull;
                }
            };

            // Start the login process in seperate thread.
            // Disable textfields
            enableComponents(false);

            // Show progressbar
            setProgressBarVisible(true);

            loginValidationThread.start();
        }

        public JPasswordField getPasswordField() {
            return passwordField;
        }

		@Override
        public Dimension getPreferredSize() {
            final Dimension dim = super.getPreferredSize();
            dim.height = 400;
            dim.width = 600;
            return dim;
        }

        public void useSSO(boolean use) {
            if (use) {
                usernameLabel.setVisible(true);
                usernameField.setVisible(true);

                passwordLabel.setVisible(false);
                passwordField.setVisible(false);

                savePasswordBox.setVisible(false);

                accountLabel.setVisible(true);
                accountNameLabel.setVisible(true);

                serverField.setVisible(true);

                autoLoginBox.setVisible(true);
                serverLabel.setVisible(true);
		loginAsInvisibleBox.setVisible(true);

                headerLabel.setVisible(true);

                if(localPref.getDebug()) {
                    System.setProperty("java.security.krb5.debug","true");
                    System.setProperty("sun.security.krb5.debug","true");
                } else {
                    System.setProperty("java.security.krb5.debug","false");
                    System.setProperty("sun.security.krb5.debug","false");
                }

                String ssoMethod = localPref.getSSOMethod();
                if(!ModelUtil.hasLength(ssoMethod)) {
                    ssoMethod = "file";
                }

                System.setProperty("javax.security.auth.useSubjectCredsOnly", "false");
                GSSAPIConfiguration config = new GSSAPIConfiguration( ssoMethod.equals("file") );
                Configuration.setConfiguration(config);

                LoginContext lc;
                String princName = localPref.getLastUsername();
                String princRealm = null;
                try {
                    lc = new LoginContext("com.sun.security.jgss.krb5.initiate");
                    lc.login();
                    Subject mySubject = lc.getSubject();


                    for (Principal p : mySubject.getPrincipals()) {
                        //TODO: check if principal is a kerberos principal first...
                        String name = p.getName();
                        int indexOne = name.indexOf("@");
                        if (indexOne != -1) {
                            princName = name.substring(0, indexOne);
                            accountNameLabel.setText(name);
                            princRealm = name.substring(indexOne+1);
                        }
                        loginButton.setEnabled(true);
                    }
                }
                catch (LoginException le) {
                    Log.debug(le.getMessage());
                    accountNameLabel.setText(Res.getString("title.login.no.account"));
                    //useSSO(false);
                }

                String ssoKdc;
                if(ssoMethod.equals("dns")) {
                    if (princRealm != null) { //princRealm is null if we got a LoginException above.
                        ssoKdc = getDnsKdc(princRealm);
                        System.setProperty("java.security.krb5.realm",princRealm);
                        System.setProperty("java.security.krb5.kdc",ssoKdc);
                    }
                } else if(ssoMethod.equals("manual")) {
                    princRealm = localPref.getSSORealm();
                    ssoKdc = localPref.getSSOKDC();
                    System.setProperty("java.security.krb5.realm",princRealm);
                    System.setProperty("java.security.krb5.kdc",ssoKdc);
                } else {
                    //Assume "file" method.  We don't have to do anything special,
                    //java takes care of it for us. Unset the props if they are set
                    System.clearProperty("java.security.krb5.realm");
                    System.clearProperty("java.security.krb5.kdc");
                }

                String userName = localPref.getLastUsername();
                if (ModelUtil.hasLength(userName)) {
                    usernameField.setText(userName);
                } else {
                    usernameField.setText(princName);
                }
            }
            else {
                autoLoginBox.setVisible(true);
                usernameField.setVisible(true);
                passwordField.setVisible(true);
                savePasswordBox.setVisible(true);
                usernameLabel.setVisible(true);
                passwordLabel.setVisible(true);
                serverLabel.setVisible(true);
                serverField.setVisible(true);
		loginAsInvisibleBox.setVisible(true);

                headerLabel.setVisible(false);
                accountLabel.setVisible(false);
                serverNameLabel.setVisible(false);
                accountNameLabel.setVisible(false);

                Configuration.setConfiguration(null);

                validateDialog();
            }


        }

        /**
         * Login to the specified server using username, password, and workgroup.
         * Handles error representation as well as logging.
         *
         * @return true if login was successful, false otherwise
         */
        private boolean login() {
           final SessionManager sessionManager
			  	= SparkManager.getSessionManager();
           boolean hasErrors = false;
           String errorMessage = null;

		  localPref.setLoginAsInvisible(loginAsInvisibleBox.isSelected());

		  // check for internal network, and use different login protocol
		  if (isInternal()) {
			  return internalNetLogin();
		  }

            // Handle specified Workgroup
            String serverName = getServerName();

            if (!hasErrors) {
                localPref = SettingsManager.getLocalPreferences();
                if (localPref.isDebuggerEnabled()) {
                    XMPPConnection.DEBUG_ENABLED = true;
                }

                SmackConfiguration.setPacketReplyTimeout(localPref.getTimeOut() * 1000);

                // Get connection
                try {
                	ConnectionConfiguration config = retrieveConnectionConfiguration();
                  connection = new XMPPConnection(config, this);
                    //If we want to use the debug version of smack, we have to check if
                    //we are on the dispatch thread because smack will create an UI
		    if (localPref.isDebuggerEnabled()) {
			if (EventQueue.isDispatchThread()) {
			    connection.connect();
			} else {
			    EventQueue.invokeAndWait(new Runnable() {

				@Override
				public void run() {
				    try {
					connection.connect();
				    } catch (XMPPException e) {
					Log.error("connection error",e);
				    }

				}
			    });
			}
		    } else {
			connection.connect();
		    }

		   // see whether we want to log in anonymously or not
		   if (isAnonymous()) {
			   // login anonymously
			   connection.loginAnonymously();
			   sessionManager.initializeSession(connection, connection.getConnectionID(), "boguspassword");
			   sessionManager.setServerAddress(connection.getServiceName());
			   sessionManager.setJID(connection.getUser());
		   }
		   else {
                    String resource = localPref.getResource();
            if (Default.getBoolean("HOSTNAME_AS_RESOURCE")) {
            try {
    			    resource = InetAddress.getLocalHost().getHostName();
    			} catch(UnknownHostException e) {
    			    Log.error("unable to retrieve hostname",e);
    			}
            } else if (localPref.isUseHostnameAsResource()) {
			try {
			    resource = InetAddress.getLocalHost().getHostName();
			} catch(UnknownHostException e) {
			    Log.error("unable to retrieve hostname",e);
			}
		    } else if (Default.getBoolean("VERSION_AS_RESOURCE")) {
		    	resource = Default.getString(Default.APPLICATION_NAME) + " " + JiveInfo.getVersion() + "." + Default.getString(Default.BUILD_NUMBER);
		    } else if (localPref.isUseVersionAsResource()) {
		    	resource = Default.getString(Default.APPLICATION_NAME) + " " + JiveInfo.getVersion() + "." + Default.getString(Default.BUILD_NUMBER);
		    }
                    connection.login(getLoginUsername(), getLoginPassword(),
                	    org.jivesoftware.spark.util.StringUtils.modifyWildcards(resource).trim());

                    sessionManager.setServerAddress(connection.getServiceName());
                    sessionManager.initializeSession(connection, getLoginUsername(), getLoginPassword());
                    sessionManager.setJID(connection.getUser());
					}
           }
                catch (Exception xee) {
                    if (!loginDialog.isVisible()) {
                        loginDialog.setVisible(true);
                    }
                    if (xee instanceof XMPPException) {

                       XMPPException xe = (XMPPException)xee;
                       final XMPPError error = xe.getXMPPError();
                       int errorCode = 0;
                       if (error != null) {
                           errorCode = error.getCode();
                       }
                       if (errorCode == 401) {
                           errorMessage = Res.getString("message.invalid.username.password");
                       }
                       else if (errorCode == 502 || errorCode == 504) {
                           errorMessage = Res.getString("message.server.unavailable");
                       }
                       else if (errorCode == 409) {
                           errorMessage = Res.getString("label.conflict.error");
                       }
			else if (errorCode == 403) {
				errorMessage = "Server does not allow anonymous logins";
			}
                       else {
                           errorMessage = Res.getString("message.unrecoverable.error");
                       }
                   }
                   else {
                       errorMessage = SparkRes.getString(SparkRes.UNRECOVERABLE_ERROR);
                   }

                    // Log Error
                    Log.warning("Exception in Login:", xee);
                    hasErrors = true;
                }
            }

            if (hasErrors) {

            	final String finalerrorMessage = errorMessage;
               EventQueue.invokeLater(new Runnable() {

     					@Override
     					public void run()
     					{
     							progressBar.setVisible(false);
     							//progressBar.setIndeterminate(false);

     							// Show error dialog
     							UIManager.put("OptionPane.okButtonText", Res.getString("ok"));
     							if (loginDialog.isVisible()) {
     								if (!localPref.isSSOEnabled()) {
     	                        JOptionPane.showMessageDialog(loginDialog, finalerrorMessage, Res.getString("title.login.error"),
     	                                JOptionPane.ERROR_MESSAGE);
     								}
     								else {
     	                        JOptionPane.showMessageDialog(loginDialog, Res.getString("title.advanced.connection.sso.unable"), Res.getString("title.login.error"),
     	                                JOptionPane.ERROR_MESSAGE);
     	                        //useSSO(false);
     	                        //localPref.setSSOEnabled(false);
     								}
     							}
     						}
     				});

                setEnabled(true);
                return false;
            }

            // Since the connection and workgroup are valid. Add a ConnectionListener
            connection.addConnectionListener(SparkManager.getSessionManager());
            //Initialize chat state notification mechanism in smack
            ChatStateManager.getInstance(SparkManager.getConnection());

            // Persist information
            localPref.setLastUsername(getLoginUsername());

            // Check to see if the password should be saved.
            if (savePasswordBox.isSelected()) {
                try {
                    localPref.setPasswordForUser(getBareJid(), getPassword());
                }
                catch (Exception e) {
                    Log.error("Error encrypting password.", e);
                }
            }

            localPref.setSavePassword(savePasswordBox.isSelected());
            localPref.setAutoLogin(autoLoginBox.isSelected());

//            if (localPref.isSSOEnabled()) {
//                localPref.setAutoLogin(true);
//            }

            localPref.setServer(serverField.getText());


            SettingsManager.saveSettings();

            return !hasErrors;
        }

        /**
         * Login to the specified external network using the host/port/domain
		 * in the specified config.
         * @return true if login was successful, false otherwise
         */
        private boolean internalNetLogin() {
          final SessionManager sessionManager
		  	= SparkManager.getSessionManager();
          boolean hasErrors = false;
          String errorMessage = null;
		  if (m_OFShost == null || m_OFShost.isEmpty()) {
			  Log.error("Missing OFS host designation, cannot connect");
			  return false;
		  }
		  if (m_OFSdomain == null || m_OFSdomain.isEmpty()) {
			  Log.error("Missing OFS domain designation, cannot connect");
			  return false;
		  }
		  if (m_OFSport <= 0) {
			  Log.error("Missing OFS port designation, cannot connect");
			  return false;
		  }
          if (localPref.isDebuggerEnabled()) {
          	XMPPConnection.DEBUG_ENABLED = true;
          }
          SmackConfiguration.setPacketReplyTimeout(30000);
		  //SmackConfiguration.setPacketReplyTimeout(localPref.getTimeOut() * 1000);

	/*
	   Log.error("Connecting to " + m_OFSdomain + " at " + m_OFShost
				   + ":" + m_OFSport);
	 */
	   ConnectionConfiguration cc
		   = new ConnectionConfiguration(m_OFShost, m_OFSport, m_OFSdomain);
	   cc.setCompressionEnabled(localPref.isCompressionEnabled());
	   cc.setNotMatchingDomainCheckEnabled(true);
	   cc.setRosterLoadedAtLogin(!isAnonymous());
	   cc.setSecurityMode(ConnectionConfiguration.SecurityMode.required);
	   cc.setSelfSignedCertificateEnabled(true);
	   cc.setSendPresence(false);
	   cc.setReconnectionAllowed(true);
      	connection = new XMPPConnection(cc, this);

	   // get connection
	   try {
		   /* If we want to use the debug version of smack, we have to
		    * check if we are on the dispatch thread because smack will
		    * create its own separate UI.
		    */
		   if (localPref.isDebuggerEnabled()) {
			   if (EventQueue.isDispatchThread()) {
				   connection.connect();
			   }
			   else {
				   EventQueue.invokeAndWait(new Runnable() {
					   @Override
					   public void run() {
						   try {
							   connection.connect();
						   }
						   catch (XMPPException e) {
							   Log.error("connection error", e);
						   }
					   }
				   });
			   }
		   }
		   else {
			   connection.connect();
		   }

		   // we require a secure connection on the private networks
		   if (!connection.isUsingTLS()) {
			   connection.disconnect();
			   Log.error("Could not establish TLS connection to server");
			   return false;
		   }


		   // see whether we want to log in anonymously or not
		   if (isAnonymous()) {
			   // login anonymously
			   connection.loginAnonymously();
			   sessionManager.initializeSession(connection, connection.getConnectionID(), "boguspassword");
			   sessionManager.setServerAddress(connection.getServiceName());
			   sessionManager.setJID(connection.getUser());
		   }
		   else {
			   // login normally with specified username and password
			   String resource = localPref.getResource();
		    	if (Default.getBoolean("VERSION_AS_RESOURCE")) {
		    		resource = Default.getString(Default.APPLICATION_NAME) + " " + JiveInfo.getVersion() + "." + Default.getString(Default.BUILD_NUMBER);
		    	} else if (Default.getBoolean("HOSTNAME_AS_RESOURCE")) {
            	try {
    			    	resource = InetAddress.getLocalHost().getHostName();
    				} catch(UnknownHostException e) {
    			    	Log.error("unable to retrieve hostname",e);
    				}
				}
			   connection.login(getLoginUsername(), getLoginPassword(), org.jivesoftware.spark.util.StringUtils.modifyWildcards(resource).trim()); 
			   sessionManager.setServerAddress(connection.getServiceName());
			   sessionManager.initializeSession(connection, getLoginUsername(), getLoginPassword());
			   sessionManager.setJID(connection.getUser());
		   }
	   }
            catch (Exception xee) {
            	if (!loginDialog.isVisible()) {
              		loginDialog.setVisible(true);
               }
               if (xee instanceof XMPPException) {
               	XMPPException xe = (XMPPException)xee;
                  final XMPPError error = xe.getXMPPError();
                	int errorCode = 0;
               	if (error != null) {
                  	errorCode = error.getCode();
                  }
                  if (errorCode == 401) {
                  	errorMessage
								= Res.getString("message.invalid.username.password");
                  }
                  else if (errorCode == 502 || errorCode == 504) {
                  	errorMessage = Res.getString("message.server.unavailable");
                  }
                  else if (errorCode == 409) {
                  	errorMessage = Res.getString("label.conflict.error");
                  }
				 else if (errorCode == 403) {
					 errorMessage = "Server does not allow anonymous logins";
				 }
                  else {
                  	errorMessage
								= Res.getString("message.unrecoverable.error");
                  }
               }
               else {
               	errorMessage
							= SparkRes.getString(SparkRes.UNRECOVERABLE_ERROR);
               }

               // Log Error
               Log.warning("Exception in Login:", xee);
               hasErrors = true;
            }
            if (hasErrors) {
            	final String finalerrorMessage = errorMessage;
               EventQueue.invokeLater(new Runnable() {
     					@Override
     					public void run() {
     						progressBar.setVisible(false);

     						// Show error dialog
     						if (loginDialog.isVisible()) {
     							if (!localPref.isSSOEnabled()) {
     	                  	JOptionPane.showMessageDialog(loginDialog,
																			finalerrorMessage,
															Res.getString("title.login.error"),
     	                                					JOptionPane.ERROR_MESSAGE);
     							}
     							else {
     	                  	JOptionPane.showMessageDialog(loginDialog,
										Res.getString("title.advanced.connection.sso.unable"),
															Res.getString("title.login.error"),
     	                                			JOptionPane.ERROR_MESSAGE);
     							}
     						}
     					}
     				});

               setEnabled(true);
               return false;
            }

            // Since the connection and workgroup are valid, add a
				// ConnectionListener
            connection.addConnectionListener(sessionManager);

		  	SmackConfiguration.setPacketReplyTimeout(localPref.getTimeOut() * 1000);

            //Initialize chat state notification mechanism in smack
            ChatStateManager.getInstance(SparkManager.getConnection());

	   if (!isAnonymous()) {
		   // Persist information
		   localPref.setLastUsername(getLoginUsername());

		   // Check to see if the password should be saved.
		   if (savePasswordBox.isSelected()) {
			    try {
				    localPref.setPasswordForUser(getBareJid(), getPassword());
			    }
			    catch (Exception e) {
				     Log.error("Error encrypting password.", e);
			    }
		   }

		   localPref.setSavePassword(savePasswordBox.isSelected());
		   localPref.setAutoLogin(autoLoginBox.isSelected());
		   localPref.setServer(serverField.getText());
		   SettingsManager.saveSettings();
	   }
       return !hasErrors;
	}

	public void handle(Callback[] callbacks) throws IOException  {
            for (Callback callback : callbacks) {
                if (callback instanceof NameCallback) {
                    NameCallback ncb = (NameCallback) callback;
                    ncb.setName(getLoginUsername());
                } else if (callback instanceof PasswordCallback) {
                    PasswordCallback pcb = (PasswordCallback) callback;
                    pcb.setPassword(getPassword().toCharArray());
                } else {
                    Log.error("Unknown callback requested: " + callback.getClass().getSimpleName());
                }
            }
        }

	   /**
		* load the properties related to PageKite gateways to the OFS
		*/
	   protected void loadGateways() {
		   // extract the expected values from configuration.properties
		   String ppp = ConfigurationRes.getString("OFShost");
		   String defOFShost = null;
		   if (ppp != null) {
			   defOFShost = new String(ppp);
		   }
		   else {
			   Log.error("Missing OFS hostname property, using default");
		   }
		   int defOFSport = 0;
		   ppp = ConfigurationRes.getString("OFSport");
		   if (ppp != null) {
			   Integer pnum = null;
			   try {
				   pnum = Integer.parseInt(ppp);
				   defOFSport = pnum.intValue();
			   }
			   catch (NumberFormatException nfe) {
				   Log.error("Invalid OFS port " + ppp, nfe);
			   }
		   }
		   else {
			   Log.error("Missing OFS port property, using default");
		   }
		   String xmpp = ConfigurationRes.getString("OFSxmpp");
		   String defOFSXMPP = defOFShost;
		   if (xmpp != null && !xmpp.isEmpty()) {
			   defOFSXMPP = new String(xmpp);
		   }
		   else {
			   Log.error("Missing OFS domain property, using default (== host)");
		   }

		   // put the default into the available OFS list
		   OFSPathDescriptor defOFS
			   = new OFSPathDescriptor(defOFSXMPP, defOFShost, defOFSport);
		   if (defOFS.m_IsValid) {
			   m_AvailOFSHosts.add(defOFS);
		   }

		   /* Now, if any additional OFSen are specified (as OFShost01,
			* OFShost02, plus matching OFSport01, OFSport02, etc.), add them
			* to the list.
			*/
		   int maxConfig = 10;
		   ppp = ConfigurationRes.getString("OFSmaxConfig");
		   if (ppp != null) {
			   Integer cnum = null;
			   try {
				   cnum = Integer.parseInt(ppp);
				   maxConfig = cnum.intValue();
			   }
			   catch (NumberFormatException nfe) {
				   Log.error("Invalid OFSmaxConfig value " + ppp, nfe);
			   }
		   }
		   for (int ofs = 1; ofs < maxConfig; ofs++) {
			   String hostProp = "OFShost0" + ofs;
			   String portProp = "OFSport0" + ofs;
			   String xmppProp = "OFSxmpp0" + ofs;
			   String ofsHost = ConfigurationRes.getString(hostProp);
			   String ofsPort = ConfigurationRes.getString(portProp);
			   String ofsXmpp = ConfigurationRes.getString(xmppProp);
			   if (ofsXmpp != null && ofsHost != null && ofsPort != null) {
				   // try to make an address out of this
				   OFSPathDescriptor ofsAddr = null;
				   Integer pNum = null;
				   try {
					   pNum = Integer.parseInt(ofsPort);
					   ofsAddr
						   = new OFSPathDescriptor(ofsXmpp, ofsHost, pNum.intValue());
					   if (ofsAddr.m_IsValid) {
						   m_AvailOFSHosts.add(ofsAddr);
					   }
				   }
				   catch (NumberFormatException nfe) {
					   Log.error("Invalid port, " + ofsPort, nfe);
				   }
				   catch (IllegalArgumentException iae) {
					   Log.error("Defined OFS is not valid: " + ofsXmpp + ":"
								   + ofsHost + ":" + ofsPort, iae);
				   }
			   }
			   else {
				   Log.error("No config found for OFS" + ofs);
				   break;
			   }
		   }

		   // supply default if we have none defined
		   if (m_AvailOFSHosts.isEmpty()) {
			   defOFS
			   	= new OFSPathDescriptor(m_OFSdomain, m_OFShost, m_OFSport);
			   m_AvailOFSHosts.add(defOFS);
		   }

		   // take first OFS gateway added to list as default
		   m_OFSHostIndex = 0;
		   defOFS = m_AvailOFSHosts.get(m_OFSHostIndex);
		   InetSocketAddress defaultOFS = defOFS.m_AccessPath;
		   m_OFShost = defaultOFS.getHostName();
		   m_OFSport = defaultOFS.getPort();
		   m_OFSdomain = defOFS.m_XMPPDomain;
	   }
   }

	/**
	 * inner class to describe an available OFS and its connect path
	 */
      protected final class OFSPathDescriptor {
	      /**
	       * the XMPP domain of the OFS server
	       */
	      public String		m_XMPPDomain;

	      /**
	       * the path to the OFS, as a hostname and port
	       */
	      public InetSocketAddress	m_AccessPath;

	      /**
	       * whether the OFS's endpoint appeared valid
	       */
	      public boolean		m_IsValid;

	      /**
	       * constructor
	       * @param domain the XMPP domain
	       * @param host the hostname of the OFS's public endpoint
	       * @param port the port number @host accessing the OFS's c2s listen port
	       */
	      public OFSPathDescriptor(String domain, String host, int port) {
		      if (domain != null && !domain.isEmpty()) {
			      m_XMPPDomain = new String(domain);
			      try {
				      m_AccessPath = new InetSocketAddress(host, port);
				      m_IsValid = true;
			      }
			      catch (IllegalArgumentException iae) {
				      Log.error("OFS host:port is not valid", iae);
			      }
		      }
	      }
      }

    /**
     * If the user quits, just shut down the
     * application.
     */
    private void quitLogin() {
        System.exit(1);
    }

    /**
     * Initializes Spark and initializes all plugins.
     */
    private void startSpark() {
        // Invoke the MainWindow.
       try
		{
			EventQueue.invokeLater(new Runnable() {
			 	public void run() {
			      final MainWindow mainWindow = MainWindow.getInstance();


			      /*
			      if (tray != null) {
			          // Remove trayIcon
			          tray.removeTrayIcon(trayIcon);
			      }
			      */
			      // Creates the Spark  Workspace and add to MainWindow
			      Workspace workspace = Workspace.getInstance();

			      LayoutSettings settings = LayoutSettingsManager.getLayoutSettings();
			      int x = settings.getMainWindowX();
			      int y = settings.getMainWindowY();
			      int width = settings.getMainWindowWidth();
			      int height = settings.getMainWindowHeight();

			      LocalPreferences pref = SettingsManager.getLocalPreferences();
			      if (pref.isDockingEnabled()) {
			          JSplitPane splitPane = mainWindow.getSplitPane();
			          workspace.getCardPanel().setMinimumSize(null);
			          splitPane.setLeftComponent(workspace.getCardPanel());
			          SparkManager.getChatManager().getChatContainer().setMinimumSize(null);
			          splitPane.setRightComponent(SparkManager.getChatManager().getChatContainer());
			          int dividerLoc = settings.getSplitPaneDividerLocation();
			          if (dividerLoc != -1) {
			              mainWindow.getSplitPane().setDividerLocation(dividerLoc);
			          }
			          else {
			              mainWindow.getSplitPane().setDividerLocation(240);
			          }

			          mainWindow.getContentPane().add(splitPane, BorderLayout.CENTER);
			      }
			      else {
			          mainWindow.getContentPane().add(workspace.getCardPanel(), BorderLayout.CENTER);
			      }

			      if (x == 0 && y == 0) {
			          // Use Default size
			          mainWindow.setSize(560, 590);

			          // Center Window on Screen
			          GraphicUtils.centerWindowOnScreen(mainWindow);
			      }
			      else {
			          mainWindow.setBounds(x, y, width, height);
			      }

			      if (loginDialog.isVisible()) {
			          mainWindow.setVisible(true);
			      }

			      loginDialog.setVisible(false);

			      // Build the layout in the workspace
			      workspace.buildLayout();
			 	}
			 });
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}


    }

    /**
     * Updates System properties with Proxy configuration.
     *
     * @throws Exception thrown if an exception occurs.
     */
    private void updateProxyConfig() throws Exception {
        if (ModelUtil.hasLength(Default.getString(Default.PROXY_PORT)) && ModelUtil.hasLength(Default.getString(Default.PROXY_HOST))) {
            String port = Default.getString(Default.PROXY_PORT);
            String host = Default.getString(Default.PROXY_HOST);
            System.setProperty("socksProxyHost", host);
            System.setProperty("socksProxyPort", port);
            return;
        }

        boolean proxyEnabled = localPref.isProxyEnabled();
        if (proxyEnabled) {
            String host = localPref.getHost();
            String port = localPref.getPort();
            String username = localPref.getProxyUsername();
            String password = localPref.getProxyPassword();
            String protocol = localPref.getProtocol();

            if (protocol.equals("SOCKS")) {
                System.setProperty("socksProxyHost", host);
                System.setProperty("socksProxyPort", port);

                if (ModelUtil.hasLength(username) && ModelUtil.hasLength(password)) {
                    System.setProperty("java.net.socks.username", username);
                    System.setProperty("java.net.socks.password", password);
                }
            }
            else {
                System.setProperty("http.proxyHost", host);
                System.setProperty("http.proxyPort", port);
                System.setProperty("https.proxyHost", host);
                System.setProperty("https.proxyPort", port);


                if (ModelUtil.hasLength(username) && ModelUtil.hasLength(password)) {
                    System.setProperty("http.proxyUser", username);
                    System.setProperty("http.proxyPassword", password);
                }

            }
        }
    }

    /**
     * Defines the background to use with the Login panel.
     */
    public class LoginBackgroundPanel extends JPanel {
		private static final long serialVersionUID = -2449309600851007447L;
		final ImageIcon icons = Default.getImageIcon(Default.LOGIN_DIALOG_BACKGROUND_IMAGE);

        /**
         * Empty constructor.
         */
        public LoginBackgroundPanel() {

        }

        /**
         * Uses an image to paint on background.
         *
         * @param g the graphics.
         */
        public void paintComponent(Graphics g) {
            Image backgroundImage = icons.getImage();
            double scaleX = getWidth() / (double)backgroundImage.getWidth(null);
            double scaleY = getHeight() / (double)backgroundImage.getHeight(null);
            AffineTransform xform = AffineTransform.getScaleInstance(scaleX, scaleY);
            ((Graphics2D)g).drawImage(backgroundImage, xform, this);
        }
    }

    /**
     * The image panel to display the Spark Logo.
     */
    public class ImagePanel extends JPanel {

		private static final long serialVersionUID = -1778389077647562606L;
		private final ImageIcon icons = Default.getImageIcon(Default.MAIN_IMAGE);

        /**
         * Uses the Spark logo to paint as the background.
         *
         * @param g the graphics to use.
         */
        public void paintComponent(Graphics g) {
            final Image backgroundImage = icons.getImage();
            final double scaleX = getWidth() / (double)backgroundImage.getWidth(null);
            final double scaleY = getHeight() / (double)backgroundImage.getHeight(null);
            AffineTransform xform = AffineTransform.getScaleInstance(scaleX, scaleY);
            ((Graphics2D)g).drawImage(backgroundImage, xform, this);
        }

        public Dimension getPreferredSize() {
            final Dimension size = super.getPreferredSize();
            size.width = icons.getIconWidth();
            size.height = icons.getIconHeight();
            return size;
        }
    }

    /**
     * Checks for historic Spark settings and upgrades the user.
     *
     * @throws Exception thrown if an error occurs.
     */
    private void checkForOldSettings() throws Exception {
        // Check for old settings.xml
        File settingsXML = new File(Spark.getSparkUserHome(), "/settings.xml");
        if (settingsXML.exists()) {
            SAXReader saxReader = new SAXReader();
            Document pluginXML;
            try {
                pluginXML = saxReader.read(settingsXML);
            }
            catch (DocumentException e) {
                Log.error(e);
                return;
            }

            List<?> plugins = pluginXML.selectNodes("/settings");
            for (Object plugin1 : plugins) {
                Element plugin = (Element) plugin1;

                String username = plugin.selectSingleNode("username").getText();
                localPref.setLastUsername(username);

                String server = plugin.selectSingleNode("server").getText();
                localPref.setServer(server);

                String autoLogin = plugin.selectSingleNode("autoLogin").getText();
                localPref.setAutoLogin(Boolean.parseBoolean(autoLogin));

                String savePassword = plugin.selectSingleNode("savePassword").getText();
                localPref.setSavePassword(Boolean.parseBoolean(savePassword));

                String password = plugin.selectSingleNode("password").getText();
                localPref.setPasswordForUser(username+"@"+server, password);

                SettingsManager.saveSettings();
            }

            // Delete settings File
            settingsXML.delete();
        }
    }

    /**
     * Use DNS to lookup a KDC
     * @param realm The realm to look up
     * @return the KDC hostname
     */
    private String getDnsKdc(String realm) {
        //Assumption: the KDC will be found with the SRV record
        // _kerberos._udp.$realm
        try {
            Hashtable<String,String> env= new Hashtable<String,String>();
            env.put("java.naming.factory.initial","com.sun.jndi.dns.DnsContextFactory");
            DirContext context = new InitialDirContext(env);
            Attributes dnsLookup = context.getAttributes("_kerberos._udp."+realm, new String[]{"SRV"});

            ArrayList<Integer> priorities = new ArrayList<Integer>();
            HashMap<Integer,List<String>> records = new HashMap<Integer,List<String>>();
            for (Enumeration<?> e = dnsLookup.getAll() ; e.hasMoreElements() ; ) {
                Attribute record = (Attribute)e.nextElement();
                for (Enumeration<?> e2 = record.getAll() ; e2.hasMoreElements() ; ) {
                    String sRecord = (String)e2.nextElement();
                    String [] sRecParts = sRecord.split(" ");
                    Integer pri = Integer.valueOf(sRecParts[0]);
                    if(priorities.contains(pri)) {
                        List<String> recs = records.get(pri);
                        if(recs == null) recs = new ArrayList<String>();
                        recs.add(sRecord);
                    } else {
                        priorities.add(pri);
                        List<String> recs = new ArrayList<String>();
                        recs.add(sRecord);
                        records.put(pri,recs);
                    }
                }
            }
            Collections.sort(priorities);
            List<String> l = records.get(priorities.get(0));
            String toprec = l.get(0);
            String [] sRecParts = toprec.split(" ");
            return sRecParts[3];
        } catch (NamingException e){
            return "";
        }
    }

    protected String getLoginUsername() {
        return loginUsername;
    }

    protected void setLoginUsername(String loginUsername) {
        this.loginUsername = loginUsername;
    }

    protected String getLoginPassword() {
        return loginPassword;
    }

    protected void setLoginPassword(String loginPassword) {
        this.loginPassword = loginPassword;
    }

    protected String getLoginServer() {
        return loginServer;
    }

    protected void setLoginServer(String loginServer) {
        this.loginServer = loginServer;
    }
    
    protected ArrayList<String> getUsernames() {
        return _usernames;
    }

   /**
    * obtain whether our selected network connection is internal or external
    * @return true if internal
    */
   protected boolean getInternalNet() {
	    return internalNet;
   }

  /**
   * set whether our selected network connection is internal or external
   * @param internal true if internal, false if external
   */
   protected void setInternalNet(boolean internal) {
	    internalNet = internal;
   }

   /**
    * obtain the gateway config for an internal network
    * @return the configuration, or null if unset or inapplicable
    */
   protected OFSPathDescriptor getGtwyConfig() {
	   if (getInternalNet() && gtwyConfig != null) {
		  return gtwyConfig;
	   }
	   return null;
   }

   /**
    * register the gateway config to be used
    * @param config the configuration to connect to for an internal network
    */
   protected void setGtwyConfig(OFSPathDescriptor config) {
	   gtwyConfig = config;
   }
}
