/*
 * last modified---
 * 	08-05-14/jet/ Spark plugin version
 * 	06-07-13/jet/ SilentVault client version
 *
 * purpose---
 * 	provide a cell renderer for the expiration column header (and others)
 */

package org.silentvault.spark.ui;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Font;
import java.awt.Color;
import javax.swing.JTable;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.table.TableCellRenderer;


/**
 * This class is used as the renderer for voucher table header elements.
 * It is needed because we want right-justification on them.
 *
 * @author Kevin Wilkerson
 */
public final class ExpColRenderer extends JPanel implements TableCellRenderer {
	// data members (none)

	// methods
	/**
	 * constructor
	 */
	public ExpColRenderer() {
		super();
		setLayout(new GridLayout());
	}

	/**
	 * method to implement the TableCellRenderer interface
	 * @param vtab the voucher table we're rendering (ignored)
	 * @param colValue the table header label value
	 * @param isSelected whether this header is selected (ignored)
	 * @param hasFocus whether this cell has the focus (we ignore this)
	 * @param row the table row (i.e. which voucher in the model, N/A)
	 * @param column the table column (ignored)
	 */
	public Component getTableCellRendererComponent(	JTable vtab,
																	Object colValue,
																	boolean isSelected,
																	boolean hasFocus,
																	int row,
																	int column )
	{
		if (column < 0 || column > 3) {
			return null;
		}
		String header = (String) colValue;

		// set the text, alignment, and background color
		removeAll();
		JTextField head = new JTextField(header);
		head.setDropTarget(null);
		head.setHorizontalAlignment(JTextField.TRAILING);
		head.setEditable(false);
		// this color is the same as TableHeader.background in Nimbus
		head.setBackground(new Color(214, 217, 223));
		final int sz = 13;

		// on column 2 (fiat equiv) use an italic font
		Font disp = null;
		if (column == 2) {
			disp = new Font(Font.SANS_SERIF, Font.ITALIC, sz);
		}
		// on column 1 (voucher value) use a bold font
		else if (column == 1) {
			disp = new Font(Font.SANS_SERIF, Font.BOLD, sz);
		}
		// otherwise use a plain font
		else {
			disp = new Font(Font.SANS_SERIF, Font.PLAIN, sz);
		}
		head.setFont(disp);
		add(head);

		return this;
	}
}
