/*
 * last modified---
 * 	08-06-14/jet/ Spark plugin version
 * 	06-14-13/jet/ new
 *
 * purpose---
 * 	provide a public class to record asset configuration details
 */

package org.silentvault.spark.ui.svx;


/**
 * Class to provide SVX asset configuration description.  An array of these will
 * exist in the XHomePane, populated from a REP_config message from the SVX.
 *
 * @author Justin Turrell
 */
public final class SVXAsset {
	/**
	 * name of the voucher asset type (such as S[BS]-S[0-6]).
	 * also used as key in hash table storage
	 */
	public String						m_AssetName;

	/**
	 * minimum amount of this asset which can be exchanged at this SVX.
	 * (if this is 0.0, then the asset is not tradeable)
	 */
	public double						m_MinExchange;

	/**
	 * commission rate (%) charged by SVX on exchanges (paid by both parties).
	 * (irrelevant if m_MinExchange == 0.0)
	 */
	public float						m_ExchangeCommission;

	/**
	 * minimum voucher which is sold for external payments at this SVX.
	 * (if this is 0.0, then the asset is not sold here)
	 */
	public double						m_MinSale;

	/**
	 * colon-separated list of external currencies accepted from buyers
	 */
	public String						m_SaleCurrencies;

	/**
	 * sale commission rate (%) charged by SVX on sales (paid by buyer).
	 * (irrelevant if m_MinSale == 0.0)
	 */
	public float						m_SaleCommission;

	/**
	 * minimum voucher which is repurchased for external payments at this SVX.
	 * (if this is 0.0, then the asset is not redeemable here)
	 */
	public double						m_MinPurchase;

	/**
	 * purchase commission rate (%) charged by SVX on repurchases (deducted from
	 * proceeds paid to seller).  (irrelevant if m_MinPurchase == 0.0)
	 */
	public float						m_PurchCommission;

	/**
	 * colon-separated list of external currencies paid to sellers
	 */
	public String						m_PurchCurrencies;

	/**
	 * constructor
	 * @param name the name of the asset
	 */
	public SVXAsset(String name) {
		m_AssetName = new String(name);
		m_SaleCurrencies = m_PurchCurrencies = "";
	}

	/**
	 * provide equals() so that List.indexOf() will work correctly
	 * @param obj the object to compare it to
	 */
	@Override
	public boolean equals(Object obj) {
		// another SVXAsset is equal to this one if same asset name
		if (obj instanceof SVXAsset) {
			SVXAsset other = (SVXAsset) obj;
			if (m_AssetName.equals(other.m_AssetName)) {
				return true;
			}
		}
		return false;
	}
}
